"""
HTTP客户端,类似OpenFeign
"""

from abc import ABC, abstractmethod
import asyncio
from dataclasses import dataclass
from typing import Optional, Any
import enum
import httpx
import inspect

class MediaType(enum.Enum):
  """
  媒体类型枚举类，用于指定HTTP请求的Content-Type头
  """
  JSON = "application/json"
  FORM_URLENCODED = "application/x-www-form-urlencoded"
  MULTIPART_FORM_DATA = "multipart/form-data"
  OCTET_STREAM = "application/octet-stream"
  XML = "application/xml"
  GIF = "image/gif"
  JPEG = "image/jpeg"
  PNG = "image/png"
  HTML = "text/html"

"""
请求方法装饰器
"""

def GetMapping(path: str):
  """
  GET请求方法装饰器

  Args:
      path (str): 请求路径，例如 "/users/{user_id}"
  """
  def decorator(func):
    func._http_method = "GET"
    func._path = path
    return func
  return decorator

def PostMapping(path: str, content_type: MediaType = MediaType.JSON):
  """
  POST请求方法装饰器

  Args:
      path (str): 请求路径，例如 "/users"
  """
  def decorator(func):
    func._http_method = "POST"
    func._path = path
    func._content_type = content_type.value
    return func
  return decorator

def PutMapping(path: str, content_type: MediaType = MediaType.JSON):
  """
  PUT请求方法装饰器

  Args:
      path (str): 请求路径，例如 "/users/{user_id}"
  """
  def decorator(func):
    func._http_method = "PUT"
    func._path = path
    func._content_type = content_type.value
    return func
  return decorator

def DeleteMapping(path: str, content_type: MediaType = MediaType.JSON):
  """
  DELETE请求方法装饰器

  Args:
      path (str): 请求路径，例如 "/users/{user_id}"
  """
  def decorator(func):
    func._http_method = "DELETE"
    func._path = path
    func._content_type = content_type.value
    return func
  return decorator

def PatchMapping(path: str, content_type: MediaType = MediaType.JSON):
  """
  PATCH请求方法装饰器

  Args:
      path (str): 请求路径，例如 "/users/{user_id}"
  """
  def decorator(func):
    func._http_method = "PATCH"
    func._path = path
    func._content_type = content_type.value
    return func
  return decorator


class FeignConfig(ABC):
  """
  Feign客户端配置基类，用于http请求前做额外处理
  """
  @abstractmethod
  async def pre_request(self, request: httpx.Request) -> httpx.Request:
    """
    对HTTP请求进行预处理

    Args:
        request (httpx.Request): 原始HTTP请求对象

    Returns:
        httpx.Request: 处理后的HTTP请求对象
    """
    pass

# 声明式客户端装饰器
class FeignClient:
  """
  Feign客户端类，用于发送HTTP请求
  
  Args:
      base_url (str): 服务的基础URL，例如 "http://localhost:8000"。 如果使用服务名，则会自动从Nacos获取服务实例的URL
      timeout (float, optional): 请求超时时间，单位为秒。默认值为5秒。
      config (Optional[FeignConfig], optional): Feign客户端配置对象。默认值为None。
  """

  def __init__(self, base_url: str, timeout: float = 5, config: Optional[FeignConfig] = None):
    if base_url.startswith("http"):
      self.base_url = base_url
      self.service_name = None
    else:
      self.base_url = None  # 延迟解析服务名
      self.service_name = base_url
    self.timeout = timeout
    self.config = config
    
  async def _resolve_service_name(self) -> str:
    """
    解析服务名到基础URL
    确保在服务发现之前Nacos客户端已经初始化
    """
    if self.base_url:
      return self.base_url
    
    if not self.service_name:
      raise ServiceDiscoveryError("服务名未设置")
    
    # 直接使用全局Nacos客户端实例
    from fastapi_nacos.core.dependencies import get_nacos_client_no_exception
    client = get_nacos_client_no_exception()
    
    service_discovery = client.discovery
    if service_discovery is None:
      raise ServiceDiscoveryError("服务发现组件未初始化，请确保应用配置了Nacos服务发现")
    
    service_instance = await service_discovery.choose_one_instance(self.service_name)
    if service_instance is None:
      raise ServiceDiscoveryError(f"未找到服务实例: {self.service_name}")
    
    # 构建基础URL
    self.base_url = f"http://{service_instance.ip}:{service_instance.port}"
    return self.base_url

  def __call__(self, cls) -> Any:
    feign_client = self  # 保存对FeignClient实例的引用
    timeout = self.timeout
    config = self.config
    # 遍历类的所有方法
    for name, method in cls.__dict__.items():
      # 只处理被GetMapping/PostMapping/PutMapping/DeleteMapping标记的方法
      if callable(method) and not name.startswith('_') and hasattr(method, '_http_method'):
        # 提取方法的HTTP元数据
        http_method = method._http_method
        path = method._path
        content_type = getattr(method, "_content_type", MediaType.JSON.value)

        # 定义新的方法用于实现HTTP请求
        def create_feign_method(http_method, path, content_type, original_method):
          # 获取原始方法的签名，用于映射位置参数到参数名
          sig = inspect.signature(original_method)
          
          async def feign_method(self, *args, **kwargs):
            try:
              # 提取实际参数（处理位置参数和dataclass对象）
              actual_kwargs = {}
              
              # 处理位置参数，将它们映射到参数名
              if args:
                # 获取参数名列表（排除self）
                param_names = list(sig.parameters.keys())[1:]  # 排除第一个参数self
                
                # 将位置参数映射到参数名
                for i, arg in enumerate(args):
                  if i < len(param_names):
                    param_name = param_names[i]
                    actual_kwargs[param_name] = arg
              
              # 处理关键字参数
              for key, value in kwargs.items():
                # 检查是否是dataclass对象
                if hasattr(value, '__dataclass_fields__'):
                  # 提取dataclass的字段和值
                  for field_name, field in value.__dataclass_fields__.items():
                    actual_kwargs[field_name] = getattr(value, field_name)
                else:
                  # 直接使用普通参数
                  actual_kwargs[key] = value
              
              # 解析服务名（如果需要）
              base_url = await feign_client._resolve_service_name()
              
              # 构建完整URL,替换路径参数（如 /user/{id}）
              url = path.format(**actual_kwargs)
              # 构造请求参数
              request_kwargs = {}
              
              # 使用传入的 content_type 参数
              request_headers = {"Content-Type": content_type}
              request_kwargs["headers"] = request_headers
              if http_method == "GET":
                # GET请求的查询参数
                request_kwargs["params"] = actual_kwargs
              elif http_method == "POST" or http_method == "PUT" or http_method == "PATCH":
                if content_type == MediaType.JSON.value:
                  # 请求的JSON数据
                  request_kwargs["json"] = actual_kwargs
                elif content_type == MediaType.FORM_URLENCODED.value:
                  # 请求的表单数据
                  request_kwargs["data"] = actual_kwargs
                elif content_type == MediaType.MULTIPART_FORM_DATA.value:
                  # 请求的多部分表单数据
                  request_kwargs["files"] = actual_kwargs
              elif http_method == "DELETE":
                request_kwargs["params"] = actual_kwargs

              # 创建HTTP请求
              # 处理base_url以/结尾的情况
              if base_url.endswith('/'):
                base_url = base_url[:-1]
              if url.startswith('/'):
                url = url[1:]
              full_url = f"{base_url}/{url}"
              request = httpx.Request(http_method, full_url, **request_kwargs)
              # 应用Feign配置（如果有）
              if config:
                request = await config.pre_request(request)
              # 发送请求
              async with httpx.AsyncClient(timeout=timeout) as client:
                response = await client.send(request)
              # 检查响应状态码
              response.raise_for_status()
              
              # 根据响应的Content-Type返回不同格式的数据
              response_content_type = response.headers.get('Content-Type', '')
              
              if 'application/json' in response_content_type:
                  # 返回JSON格式数据
                  return response.json()
              elif 'text/' in response_content_type:
                  # 返回文本格式数据
                  return response.text
              else:
                  # 返回原始字节数据
                  return response.content
            except httpx.HTTPStatusError as e:
              # 处理HTTP状态错误（4xx, 5xx）
              print(f"HTTP错误: {e.response.status_code} - {e.response.text}")
              raise e
            except httpx.RequestError as e:
              # 处理请求错误（网络问题等）
              print(f"请求错误: {e}")
              raise e
          return feign_method

        new_method = create_feign_method(http_method, path, content_type, method)

        # 替换原方法为新的HTTP请求实现
        setattr(cls, name, new_method)
    # 返回增强后的类
    return cls

@dataclass
class ReqModel:
  postId: int = None
  title: str = ''
  body: str = ''
  userId: int = 1


@FeignClient(base_url="https://jsonplaceholder.typicode.com")
class TestClient:

  @GetMapping("/posts")
  async def get_posts(self) -> list:
    pass
  
  @GetMapping("/posts/{id}")
  async def get_post(self, id: int) -> dict:
    pass
  
  @GetMapping("/comments")
  async def get_comments(self, req: ReqModel) -> list:
    pass
  @GetMapping("/comments")
  async def get_comment2s(self, postId: int) -> list:
    pass

  @PostMapping("/posts")
  async def post_posts(self, req: ReqModel) -> dict:
    pass
  @PutMapping("/posts/{id}")
  async def put_posts(self, id: int, req: ReqModel) -> dict:
    pass

async def main():
  client = TestClient()
  # print('get_posts')
  # print(await client.get_posts())
  print("获取帖子ID为1的所有评论:")
  comments = await client.get_comment2s(postId=1)
  print(comments)
  print("获取帖子ID为1的所有评论:")
  comments = await client.get_comment2s(1)
  print(comments)
  # print("创建新帖子:")
  # new_post = await client.post_posts(req=ReqModel(title="新帖子", body="这是新帖子的内容"))
  # print(new_post)
  # print("更新帖子ID为1的内容:")
  # updated_post = await client.put_posts(id=1, req=ReqModel(title="更新后的帖子", body="这是更新后的帖子内容"))
  # print(updated_post)

if __name__ == "__main__":
  asyncio.run(main())


import sys, os
from loguru import logger
from fastapi_nacos.config import app_config

# 获取当前项目的绝对路径
startup_file = os.path.abspath(sys.argv[0])
root_dir = os.path.dirname(startup_file)

# 从配置中获取日志级别
log_level = app_config.get("logging.level", "DEBUG")
log_file = app_config.get("logging.file", os.path.join(root_dir, "logs", "app.log"))

# 处理相对路径和绝对路径
if log_file and not os.path.isabs(log_file):
    log_file = os.path.join(root_dir, log_file)

log_dir = os.path.dirname(log_file)
if log_dir:  # 避免空路径
    os.makedirs(log_dir, exist_ok=True)

class MyLogger:
  def __init__(self):
    self.logger = logger
    # 清空所有配置
    self.logger.remove()
    # 添加控制台输出格式
    self.logger.add(sys.stdout, level=log_level,
      format="<green>{time:YYYY-MM-DD HH:mm:ss}</green> | "
              "{process.name} | " # 进程名
              "{thread.name} | " # 线程名
              "<level>{level}</level> | "
              "<cyan>{module}</cyan>.<cyan>{function}</cyan>" # 模块名.方法名
              ":<cyan>{line}</cyan>: " # 行号
              "- <level>{message}</level>" # 日志内容
    )

    # 输出到文件的格式
    self.logger.add(
      log_file,
      level=log_level,
      rotation="100 MB", # 每个日志文件最大100MB
      retention="10 days", # 保留10天的日志文件
      encoding="utf-8",
      format="{time:YYYY-MM-DD HH:mm:ss} | "
              "{process.name} | " # 进程名
              "{thread.name} | " # 线程名
              "{level} | "
              "{module}.{function}" # 模块名.方法名
              ":{line}: " # 行号
              "- {message}" # 日志内容
    )

  def get_logger(self):
    return self.logger

log = MyLogger().get_logger()


import socket
from fastapi_nacos.utils.log_utils import log

def get_ip_address(prefer_internal: bool = True) -> str:
        """
        获取IP地址
        
        Args:
            prefer_internal: 是否优先获取内网IP
            
        Returns:
            IP地址字符串
        """
        try:
            if prefer_internal:
                # 获取内网IP
                try:
                    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
                    s.connect(('8.8.8.8', 53))
                    ip = s.getsockname()[0]
                    s.close()
                    log.info(f"获取到内网IP: {ip}")
                    return ip
                except Exception as e:
                    log.warning(f"获取内网IP失败: {e}")
            
            # 方法2：获取所有IP，排除回环地址
            hostname = socket.gethostname()
            addrs = socket.getaddrinfo(hostname, None)
            
            for addr in addrs:
                ip = addr[4][0]
                # 只返回IPv4地址，排除回环地址
                if ':' not in ip and not ip.startswith('127.'):
                    log.info(f"从主机名获取到IP: {ip}")
                    return ip
            
            # 方法3：回退到本地回环地址
            log.warning("无法获取IP地址，使用127.0.0.1")
            return "127.0.0.1"
            
        except Exception as e:
            log.error(f"获取IP地址时发生错误: {e}")
            return "127.0.0.1"


from typing import Dict, Optional, List, Callable, Any
from v2.nacos import ClientConfigBuilder, NacosNamingService, NacosConfigService
from fastapi_nacos.core.registration import ServiceRegistry
from fastapi_nacos.core.discovery import ServiceDiscovery
from fastapi_nacos.core.config import ConfigManager
from fastapi_nacos.models.service import ServiceInstance
from fastapi_nacos.models.config import ConfigListener
from fastapi_nacos.utils.log_utils import log, log_dir, log_level
from fastapi_nacos.utils.exceptions import NacosConnectionError
from fastapi_nacos.utils.app_config_utils import parse_yaml_content, AppConfig
import yaml

class NacosClientManager:
    """Nacos客户端管理器，SDK的主要入口点"""
    
    # 单例实例
    _instance: Optional['NacosClientManager'] = None
    
    def __init__(self):
        """
        初始化Nacos客户端管理器
        """
        NacosClientManager._instance = self
        print("初始化Nacos客户端管理器-----------------------------------")
        # 维护全局配置字典，用于存储解析后的Nacos配置信息
        self.all_config_dict: Dict[str, AppConfig] = {}

    async def init_registry_discovery_service(
        self,
        server_addresses: str,
        namespace: str = "",
        username: Optional[str] = None,
        password: Optional[str] = None,
    ) -> None:
        """
        初始化注册中心基础服务
        
        Args:
            server_addresses: Nacos服务器地址
            namespace: Nacos命名空间ID
            username: Nacos用户名
            password: Nacos密码
        """
        try:
            client_config = (ClientConfigBuilder()
                              .server_address(server_addresses)
                              .namespace_id(namespace)
                              .username(username)
                              .password(password)
                              .log_level(log_level)
                              .log_dir(log_dir)
                              .build()
                            )
            self.naming_service = await NacosNamingService.create_naming_service(client_config)
            self._registry = ServiceRegistry(self.naming_service, log, server_addresses, namespace, username, password)
            self._discovery = ServiceDiscovery(self.naming_service, log, server_addresses, namespace, username, password)
        except Exception as e:
            log.error(f"初始化注册中心基础服务失败: {str(e)}")
            raise NacosConnectionError(f"初始化注册中心基础服务失败: {str(e)}") from e

    @property
    def registry(self) -> ServiceRegistry:
        """获取服务注册管理器实例"""
        return self._registry

    @property
    def discovery(self) -> ServiceDiscovery:
        """获取服务发现管理器实例"""
        return self._discovery

    async def init_config_service(
        self,
        server_addresses: str,
        namespace: str = "",
        username: Optional[str] = None,
        password: Optional[str] = None,
    ) -> None:
        """
        初始化配置中心基础服务
        
        Args:
            server_addresses: Nacos服务器地址
            namespace: Nacos命名空间ID
            username: Nacos用户名
            password: Nacos密码
        """
        # 初始化配置中心基础服务
        try:
            client_config = (ClientConfigBuilder()
                              .server_address(server_addresses)
                              .namespace_id(namespace)
                              .username(username)
                              .password(password)
                              .log_level(log_level)
                              .log_dir(log_dir)
                              .build()
                            )
            self.config_service = await NacosConfigService.create_config_service(client_config)
            self._config = ConfigManager(self.config_service, log, server_addresses, namespace, username, password)
        except Exception as e:
            log.error(f"初始化配置中心基础服务失败: {str(e)}")
            raise NacosConnectionError(f"初始化配置中心基础服务失败: {str(e)}") from e
    
    @property
    def config(self) -> ConfigManager:
        """获取配置管理器实例"""
        return self._config
    
    async def fetch_and_parse_config(self, data_id: str, group: str = "DEFAULT_GROUP") -> Dict[str, Any]:
        """
        根据指定的data-id和group从Nacos服务获取配置信息并解析
        
        Args:
            data_id: 配置的data-id
            group: 配置的group，默认值为"DEFAULT_GROUP"
            
        Returns:
            解析后的配置字典
            
        Raises:
            NacosConnectionError: 配置中心服务未初始化或网络请求失败
            yaml.YAMLError: YAML解析错误
            Exception: 其他未知错误
        """
        if not hasattr(self, '_config'):
            raise NacosConnectionError("配置中心服务未初始化，请先调用init_config_service方法")
        
        try:
            # 从Nacos获取配置
            config_content = await self.config.get_config(data_id, group)
            await self.parse_config_content(data_id, group, config_content)
            
        except NacosConnectionError:
            raise
        except yaml.YAMLError:
            raise
        except Exception as e:
            log.error(f"获取配置失败: {str(e)}, data_id={data_id}, group={group}")
            raise NacosConnectionError(f"获取配置失败: {str(e)}") from e

    async def parse_config_content(self, data_id: str, group: str = "DEFAULT_GROUP", config_content: str = None):
        """解析并更新Nacos配置"""
        if not config_content:
            log.warning(f"未获取到配置: data_id={data_id}, group={group}")
            self.all_config_dict[data_id] = AppConfig({})
            return {}
        
        # 解析YAML格式的配置内容
        try:
            yaml_dict = yaml.safe_load(config_content)
            if not isinstance(yaml_dict, dict):
                log.warning(f"配置内容不是有效的YAML字典: data_id={data_id}, group={group}")
                return {}
        except yaml.YAMLError as e:
            log.error(f"YAML解析错误: {str(e)}, data_id={data_id}, group={group}")
            raise
        
        # 使用parse_yaml_content方法解析配置内容
        parsed_config = parse_yaml_content(yaml_dict)
        
        # 将解析后的配置结果更新到all_config_dict中
        # 使用data_id作为键，确保配置数据的准确性和完整性
        self.all_config_dict[data_id] = AppConfig(parsed_config)
        
        log.info(f"配置获取并解析成功: data_id={data_id}, group={group}")
        return parsed_config

    def get_config_val(self, key: str, default: Any = None) -> Any:
        """
        获取指定key的配置值
        
        Args:
            key: 配置项的键
        Returns:
            配置项的值，若不存在则返回None
        """
        for data_id, config_dict in self.all_config_dict.items():
          val = config_dict.get(key, None)
          if val:
            return val
        return default

    async def register_service(
        self,
        service_name: str,
        ip: str,
        port: int,
        group_name: str = 'DEFAULT_GROUP',
        weight: Optional[float] = 1.0,
        metadata: Dict[str, str] = {},
        cluster_name: str = '',
        ephemeral: bool = True
    ) -> str:
        """
        注册服务到Nacos
        
        Args:
            service_name: 服务名称
            ip: 服务IP地址
            port: 服务端口
            group_name: 服务分组
            weight: 服务权重
            metadata: 服务元数据
            cluster_name: 集群名称
            ephemeral: 是否为临时实例
            
        Returns:
            str: 注册的服务实例ID
        """
        if self.registry:
          instance_id = await self.registry.register_service(
              service_name=service_name,
              group_name=group_name,
              ip=ip,
              port=port,
              weight=weight,
              metadata=metadata or {},
              cluster_name=cluster_name,
              ephemeral=ephemeral
          )
          return instance_id
        else:
          log.error("Nacos注册中心客户端未初始化")
    
    async def deregister_service(
        self,
        service_name: str,
        group_name: str = "DEFAULT_GROUP",
        ip: Optional[str] = None,
        port: Optional[int] = None,
        cluster_name: str = "DEFAULT",
        ephemeral: bool = True
    ) -> bool:
        """
        从Nacos注销服务
        
        Args:
            service_name: 服务名称
            group_name: 服务分组
            ip: 服务IP地址（如果未提供，将使用注册时的IP）
            port: 服务端口（如果未提供，将使用注册时的端口）
            cluster_name: 集群名称
            ephemeral: 是否为临时实例
            
        Returns:
            bool: 注销是否成功
        """
        if self.registry:
          return await self.registry.deregister_service(
              service_name=service_name,
              group_name=group_name,
              ip=ip,
              port=port,
              cluster_name=cluster_name,
              ephemeral=ephemeral
          )
        else:
          log.error("Nacos注册中心客户端未初始化")
          return False
    
    async def get_service_instances(
        self,
        service_name: str,
        group_name: str = "DEFAULT_GROUP",
        healthy_only: bool = True,
        clusters: Optional[List[str]] = None
    ) -> List[ServiceInstance]:
        """
        获取服务实例列表
        
        Args:
            service_name: 服务名称
            group_name: 服务分组
            healthy_only: 是否只返回健康实例
            clusters: 集群列表
            
        Returns:
            List[ServiceInstance]: 服务实例列表
        """
        return await self.discovery.get_service_instances(
            service_name=service_name,
            group_name=group_name,
            healthy_only=healthy_only,
            clusters=clusters
        )
    
    async def choose_one_instance(
        self,
        service_name: str,
        group_name: str = "DEFAULT_GROUP",
        healthy_only: bool = True,
        clusters: Optional[List[str]] = None,
        strategy: str = "random"
    ) -> Optional[ServiceInstance]:
        """
        选择一个服务实例（支持负载均衡策略）
        
        Args:
            service_name: 服务名称
            group_name: 服务分组
            healthy_only: 是否只返回健康实例
            clusters: 集群列表
            strategy: 负载均衡策略，可选值: random, round_robin, weight_random
            
        Returns:
            Optional[ServiceInstance]: 选中的服务实例
        """
        return await self.discovery.choose_one_instance(
            service_name=service_name,
            group_name=group_name,
            healthy_only=healthy_only,
            clusters=clusters,
            strategy=strategy
        )
    
    async def get_config(
        self,
        data_id: str,
        group: str = "DEFAULT_GROUP",
    ) -> Optional[str]:
        """
        获取配置信息
        
        Args:
            data_id: 配置ID
            group: 配置分组
            
        Returns:
            Optional[str]: 配置内容
        """
        log.info(f"获取配置: data_id={data_id}, group={group}")
        
        return await self.config.get_config(data_id=data_id, group=group)
    
    async def add_config_listener(
        self,
        data_id: str,
        callback: Callable[[str], None],
        group: str = "DEFAULT_GROUP",
        namespace: str = "",
        content_type: str = "text"
    ) -> bool:
        """
        添加配置监听器
        
        Args:
            data_id: 配置ID
            callback: 配置变更回调函数
            group: 配置分组
            namespace: 命名空间ID
            content_type: 内容类型
            
        Returns:
            bool: 添加是否成功
        """
        listener = ConfigListener(
            data_id=data_id,
            group=group,
            namespace=namespace,
            callback=callback,
            content_type=content_type
        )
        return await self.config.add_listener(listener)

    async def config_shutdown(self):
        """
        关闭配置中心客户端
        """
        if self.config:
          await self.config.shutdown()

    @classmethod
    def get_instance(cls) -> Optional['NacosClientManager']:
        """
        获取Nacos客户端管理器的单例实例
        
        Returns:
            Optional[NacosClientManager]: Nacos客户端管理器实例
        """
        if cls._instance is None:
            cls._instance = NacosClientManager()
        return cls._instance

    @classmethod
    def get_registry_instance(cls) -> Optional['ServiceRegistry']:
        """
        获取服务注册管理器实例
        
        Returns:
            Optional[ServiceRegistry]: 服务注册管理器实例
        """
        return cls._instance.registry

    @classmethod
    def get_discovery_instance(cls) -> Optional['ServiceDiscovery']:
        """
        获取服务发现管理器实例
        
        Returns:
            Optional[ServiceDiscovery]: 服务发现管理器实例
        """
        return cls._instance.discovery



from os import name
import time
import threading
from typing import Dict, Optional
from v2.nacos import NacosNamingService
from fastapi_nacos.models.service import ServiceRegisterRequest, ServiceInstance
from fastapi_nacos.utils.exceptions import ServiceRegistrationError, HeartbeatError


class ServiceRegistry:
    """服务注册管理类"""
    
    def __init__(self, 
        naming_service: NacosNamingService, 
        logger,
        server_addresses: str,
        namespace: str = "",
        username: Optional[str] = None,
        password: Optional[str] = None):
        """
        初始化服务注册管理器
        
        Args:
            naming_service: Nacos命名服务实例
            logger: 日志记录器
            server_addresses: Nacos服务器地址
            namespace: Nacos命名空间ID
            username: Nacos用户名
            password: Nacos密码
        """
        self.naming_service = naming_service
        self.logger = logger
        self.server_addresses = server_addresses
        self.namespace = namespace
        self.username = username
        self.password = password
        self.registered_instances: Dict[str, Dict] = {}  # 存储已注册的实例信息
        self.heartbeat_threads: Dict[str, threading.Thread] = {}  # 存储心跳线程
        self.heartbeat_stop_events: Dict[str, threading.Event] = {}  # 存储心跳停止事件
        self.heartbeat_interval = 5  # 心跳间隔，单位：秒
    
    async def register_service(self,
        service_name: str,
        ip: str,
        port: int,
        group_name: str = 'DEFAULT_GROUP',
        weight: Optional[float] = 1.0,
        metadata: Dict[str, str] = {},
        cluster_name: str = '',
        ephemeral: bool = True) -> str:
        """
        注册服务到Nacos
        
        Args:
            service_name: 服务名称
            group_name: 服务分组名称
            ip: 服务实例IP地址
            port: 服务实例端口号
            weight: 服务实例权重（0-1000）
            metadata: 服务实例元数据
            cluster_name: 服务实例所在集群名称
            ephemeral: 是否为临时实例
            
        Returns:
            str: 注册的服务实例ID
        """
        try:
            self.logger.info(f"开始注册服务: {service_name}，IP: {ip}:{port}")
            # 调用Nacos客户端注册服务
            # 使用RegisterInstanceParam对象
            from v2.nacos.naming.model.naming_param import RegisterInstanceParam
            register_param = RegisterInstanceParam(
                service_name=service_name,
                group_name=group_name,
                ip=ip,
                port=port,
                weight=weight,
                metadata=metadata,
                cluster_name=cluster_name,
                ephemeral=ephemeral
            )
            result = await self.naming_service.register_instance(register_param)
            self.logger.debug(f"注册服务返回结果: {result}")
            
            # 生成实例ID（格式：ip:port:clusterName@serviceName）
            instance_id = f"{ip}:{port}:{cluster_name or 'DEFAULT'}@{service_name}"
            
            self.logger.info(f"服务注册成功: {service_name}，实例ID: {instance_id}")
            
            # 如果是临时实例，启动心跳线程
            if ephemeral:
                self._start_heartbeat(
                    instance_id=instance_id,
                    service_name=service_name,
                    ip=ip,
                    port=port,
                    group_name=group_name,
                    cluster_name=cluster_name
                )
            
            # 保存注册信息
            self.registered_instances[instance_id] = {
                "service_name": service_name,
                "ip": ip,
                "port": port,
                "group_name": group_name,
                "cluster_name": cluster_name,
                "ephemeral": ephemeral
            }
            
            return instance_id
        except Exception as e:
            self.logger.error(f"服务注册失败: {str(e)}")
            raise ServiceRegistrationError(f"服务注册失败: {str(e)}") from e
    
    async def deregister_service(
        self,
        service_name: str,
        group_name: str = "DEFAULT_GROUP",
        ip: Optional[str] = None,
        port: Optional[int] = None,
        cluster_name: str = "DEFAULT",
        ephemeral: bool = True
    ) -> bool:
        """
        从Nacos注销服务
        
        Args:
            service_name: 服务名称
            group_name: 服务分组
            ip: 服务IP地址
            port: 服务端口
            cluster_name: 集群名称
            ephemeral: 是否为临时实例
            
        Returns:
            bool: 注销是否成功
        """
        try:
            self.logger.info(f"开始注销服务: {service_name}")
            
            # 调用Nacos客户端注销服务
            # 使用DeregisterInstanceParam对象
            from v2.nacos.naming.model.naming_param import DeregisterInstanceParam
            deregister_param = DeregisterInstanceParam(
                service_name=service_name,
                group_name=group_name,
                ip=ip,
                port=port,
                cluster_name=cluster_name,
                ephemeral=ephemeral
            )
            result = await self.naming_service.deregister_instance(deregister_param)
            
            # 停止心跳线程（如果存在）
            if ephemeral:
                self._stop_heartbeat(service_name, ip, port)
            
            self.logger.info(f"服务注销成功: {service_name}")
            return result
        except Exception as e:
            self.logger.error(f"服务注销失败: {str(e)}")
            return False
    
    def _start_heartbeat(
        self,
        instance_id: str,
        service_name: str,
        ip: str,
        port: int,
        group_name: str,
        cluster_name: str
    ):
        """
        启动心跳线程
        
        注意：在新版本的Nacos SDK中，心跳由SDK内部管理，不需要手动发送心跳
        
        Args:
            instance_id: 实例ID
            service_name: 服务名称
            ip: 服务IP地址
            port: 服务端口
            group_name: 服务分组
            cluster_name: 集群名称
        """
        self.logger.info(f"服务实例 {instance_id} 心跳由Nacos SDK内部管理")
    
    def _stop_heartbeat(self, service_name: str, ip: str, port: int):
        """
        停止心跳线程
        
        注意：在新版本的Nacos SDK中，心跳由SDK内部管理，不需要手动停止心跳
        
        Args:
            service_name: 服务名称
            ip: 服务IP地址
            port: 服务端口
        """
        # 查找对应的实例ID
        instance_id = None
        for id, info in self.registered_instances.items():
            if (info["service_name"] == service_name and 
                info["ip"] == ip and 
                info["port"] == port):
                instance_id = id
                break
        
        if instance_id:
            # 移除注册信息
            if instance_id in self.registered_instances:
                del self.registered_instances[instance_id]
            
            self.logger.info(f"移除服务实例注册信息: {service_name}，实例ID: {instance_id}")
    
    def update_health_status(
        self,
        service_name: str,
        ip: str,
        port: int,
        healthy: bool,
        group_name: str = "DEFAULT_GROUP",
        cluster_name: str = "DEFAULT"
    ) -> bool:
        """
        更新服务健康状态
        
        注意：在新版本的Nacos SDK中，健康状态由SDK内部管理
        
        Args:
            service_name: 服务名称
            ip: 服务IP地址
            port: 服务端口
            healthy: 健康状态
            group_name: 服务分组
            cluster_name: 集群名称
            
        Returns:
            bool: 更新是否成功
        """
        self.logger.warning("在新版本的Nacos SDK中，健康状态由SDK内部管理，不需要手动更新")
        return True
    
    def get_registered_instances(self) -> Dict[str, Dict]:
        """
        获取已注册的服务实例列表
        
        Returns:
            Dict[str, Dict]: 已注册的服务实例信息
        """
        return self.registered_instances


# 导出主要类和功能
from fastapi_nacos.core import (
    NacosClientManager,
    nacos_lifespan,
    init_nacos_with_fastapi,
)
from fastapi_nacos.core.dependencies import (
    init_nacos_registry_discovery_client,
    get_nacos_client,
    get_service_registry,
    get_service_discovery,
    get_config_manager
)
from fastapi_nacos.models.service import ServiceInstance, ServiceRegisterRequest, ServiceInfo
from fastapi_nacos.models.config import ConfigRequest, ConfigResponse, ConfigListener
from fastapi_nacos.utils.exceptions import (
    FastApiNacosException,
    NacosConnectionError,
    ServiceRegistrationError,
    ServiceDiscoveryError,
    ConfigError,
    ConfigListenerError,
    HeartbeatError
)
from fastapi_nacos.http.http_client import (
  FeignClient,
  GetMapping,
  PostMapping,
  PutMapping,
  DeleteMapping
)

from fastapi_nacos.core.value import Value

__version__ = "0.1.0"
__all__ = [
    # 核心类
    "NacosClientManager",
    "nacos_lifespan",
    "init_nacos_with_fastapi",
    
    # 依赖注入函数
    "init_nacos_registry_discovery_client",
    "get_nacos_client",
    "get_service_registry",
    "get_service_discovery",
    "get_config_manager",
    
    # 服务模型
    "ServiceInstance",
    "ServiceRegisterRequest",
    "ServiceInfo",
    
    # 配置模型
    "ConfigRequest",
    "ConfigResponse",
    "ConfigListener",
    
    # 异常类
    "FastApiNacosException",
    "NacosConnectionError",
    "ServiceRegistrationError",
    "ServiceDiscoveryError",
    "ConfigError",
    "ConfigListenerError",
    "HeartbeatError",
    
    # HTTP客户端
    "FeignClient",
    "GetMapping",
    "PostMapping",
    "PutMapping",
    "DeleteMapping",
    
    # 配置值装饰器
    "Value"
]


class FastApiNacosException(Exception):
    """FastAPI-Nacos SDK的基础异常类"""
    pass


class NacosConnectionError(FastApiNacosException):
    """Nacos连接错误异常"""
    pass


class ServiceRegistrationError(FastApiNacosException):
    """服务注册错误异常"""
    pass


class ServiceDiscoveryError(FastApiNacosException):
    """服务发现错误异常"""
    pass


class ConfigError(FastApiNacosException):
    """配置中心错误异常"""
    pass


class ConfigListenerError(FastApiNacosException):
    """配置监听错误异常"""
    pass


class HeartbeatError(FastApiNacosException):
    """心跳发送错误异常"""
    pass


import os
import sys
from dotenv import load_dotenv

# 获取当前项目的绝对路径
startup_file = os.path.abspath(sys.argv[0])
root_dir = os.path.dirname(startup_file)

# 加载项目根目录下的.env文件
load_dotenv(os.path.join(root_dir, '.env'), override=True)

# 注册中心配置
discovery_server_addresses = os.getenv("NACOS_DISCOVERY_SERVER_ADDRESSES")
discovery_namespace = os.getenv("NACOS_DISCOVERY_NAMESPACE")
discovery_username = os.getenv("NACOS_DISCOVERY_USERNAME")
discovery_password = os.getenv("NACOS_DISCOVERY_PASSWORD")

# 配置中心配置
config_server_addresses = os.getenv("NACOS_CONFIG_SERVER_ADDRESSES")
config_namespace = os.getenv("NACOS_CONFIG_NAMESPACE")
config_username = os.getenv("NACOS_CONFIG_USERNAME")
config_password = os.getenv("NACOS_CONFIG_PASSWORD")

def get_var(var_name: str, default: str = None) -> str:
    """获取环境变量
    
    Args:
        var_name: 环境变量名
        default: 默认值
        
    Returns:
        环境变量值或默认值
    """
    return os.getenv(var_name, default)



from typing import Dict, Optional, Callable
from pydantic import BaseModel, Field


class ConfigRequest(BaseModel):
    """配置获取请求模型"""
    data_id: str
    group: str = Field(default="DEFAULT_GROUP")


class ConfigResponse(BaseModel):
    """配置获取响应模型"""
    data_id: str
    group: str
    namespace: str
    content: str
    type: Optional[str] = Field(default="text")


class ConfigListener(BaseModel):
    """配置监听器模型"""
    data_id: str
    group: str
    callback: Callable[[str], None]


from typing import Dict, Optional, List
from pydantic import BaseModel, Field


class ServiceInstance(BaseModel):
    """服务实例模型"""
    ip: str
    port: int
    service_name: str
    group_name: str = Field(default="DEFAULT_GROUP")
    weight: float = Field(default=1.0, ge=0, le=1000)
    healthy: bool = Field(default=True)
    enabled: bool = Field(default=True)
    metadata: Dict[str, str] = Field(default_factory=dict)
    cluster_name: str = Field(default="DEFAULT")
    instance_id: Optional[str] = Field(default=None)


class ServiceRegisterRequest(BaseModel):
    """服务注册请求模型"""
    service_name: str
    group_name: str = Field(default="DEFAULT_GROUP")
    ip: str
    port: int
    weight: float = Field(default=1.0, ge=0, le=1000)
    metadata: Dict[str, str] = Field(default_factory=dict)
    cluster_name: str = Field(default="DEFAULT")
    ephemeral: bool = Field(default=True)


class ServiceInfo(BaseModel):
    """服务信息模型"""
    name: str
    group_name: str
    clusters: Optional[str] = None
    cacheMillis: int = 1000
    hosts: List[ServiceInstance]
    last_ref_time: Optional[int] = None
    checksum: Optional[str] = None
    all_ip: bool = False
    reach_protection_threshold: Optional[int] = None


"""
项目配置文件解析工具
"""

import os
import sys
import yaml
import re
from fastapi_nacos.utils.env_utils import get_var
from typing import Dict, Any, Union

# 环境变量引用正则表达式: ${ENV_VAR:default_value}
ENV_VAR_PATTERN = re.compile(r'\$\{([^:}]+)(?::([^}]*))?\}')

# 配置参数引用正则表达式: ${config.key}
CONFIG_VAR_PATTERN = re.compile(r'\$\{([a-zA-Z0-9_.]+)\}')


def substitute_env_vars(value: Union[str, Dict[str, Any], Any], config_dict: Dict[str, Any] = None) -> Union[str, Dict[str, Any], Any]:
    """递归替换字符串中的环境变量引用和配置参数引用
    
    Args:
        value: 要处理的值，可以是字符串、字典或其他类型
        config_dict: 配置字典，用于解析配置参数引用
        
    Returns:
        替换后的对应值
    """
    if isinstance(value, str):
        # 首先替换配置参数引用
        def replace_config_match(match: re.Match) -> str:
            config_key = match.group(1)
            if config_dict is not None:
                # 从配置字典中查找对应的值
                if config_key in config_dict:
                    # 直接查找扁平化的键
                    return str(config_dict[config_key])
                else:
                    # 如果路径不存在，返回原字符串
                    return match.group(0)
            else:
                # 如果没有配置字典，返回原字符串
                return match.group(0)
        
        # 替换配置参数引用
        result = CONFIG_VAR_PATTERN.sub(replace_config_match, value)
        
        # 然后替换环境变量引用
        def replace_env_match(match: re.Match) -> str:
            env_var = match.group(1)
            default = match.group(2) or ''
            return get_var(env_var, default)
        
        return ENV_VAR_PATTERN.sub(replace_env_match, result)
    elif isinstance(value, dict):
        # 递归处理字典
        return {
            key: substitute_env_vars(val, config_dict)
            for key, val in value.items()
        }
    elif isinstance(value, list):
        # 递归处理列表
        return [substitute_env_vars(item, config_dict) for item in value]
    else:
        # 其他类型直接返回
        return value


class AppConfig:
    """配置对象类，提供属性访问和字典访问两种方式"""
    
    def __init__(self, config_dict: Dict[str, Any]):
        """初始化配置对象
        
        Args:
            config_dict: 配置字典
        """
        self._config = config_dict
        # 将字典转换为属性
        self._convert_dict_to_attrs(config_dict)
    
    def _convert_dict_to_attrs(self, data: Dict[str, Any], prefix: str = ""):
        """将字典递归转换为对象属性
        
        Args:
            data: 要转换的字典数据
            prefix: 属性前缀（用于嵌套结构）
        """
        for key, value in data.items():
            attr_name = f"{prefix}{key}" if prefix else key
            if isinstance(value, dict):
                # 递归处理嵌套字典
                nested_attr = AppConfig(value)
                setattr(self, attr_name, nested_attr)
            else:
                # 直接设置简单值
                setattr(self, attr_name, value)
    
    def get(self, key: str, default: Any = None) -> Any:
        """获取配置值（类似字典的get方法）
        
        Args:
            key: 配置键名，支持嵌套格式如 "db.host"
            default: 默认值
            
        Returns:
            配置值或默认值
        """
        keys = key.split('.')
        value = self._config
        
        try:
            for k in keys:
                if isinstance(value, dict):
                    value = value[k]
                elif hasattr(value, k):
                    value = getattr(value, k)
                else:
                    return default
            return value
        except (KeyError, AttributeError):
            return default
    
    def __getitem__(self, key: str) -> Any:
        """支持字典式访问
        
        Args:
            key: 配置键名
            
        Returns:
            配置值
        """
        return self.get(key)
    
    def __contains__(self, key: str) -> bool:
        """检查配置是否包含指定键
        
        Args:
            key: 配置键名
            
        Returns:
            是否包含该键
        """
        return self.get(key) is not None
    
    def to_dict(self) -> Dict[str, Any]:
        """转换为字典格式
        
        Returns:
            配置字典
        """
        return self._config
    
    def __str__(self) -> str:
        """字符串表示
        
        Returns:
            配置的字符串表示
        """
        return yaml.dump(self._config, default_flow_style=False, allow_unicode=True)

def parse_yaml_content(config_dict: Dict[str, Any]) -> Dict[str, Any]:
    """解析 YAML 内容
    
    Args:
        config_dict: 包含 YAML 内容的字典
        
    Returns:
        解析后的配置字典
    """
    # Process the config with multiple iterations to handle dependencies
    max_iterations = 10
    current_config = config_dict
    
    for iteration in range(max_iterations):
        # Create a flat lookup of the current config state
        def create_flat_lookup(cfg, prefix=''):
            lookup = {}
            for k, v in cfg.items():
                full_key = f"{prefix}.{k}" if prefix else k
                if isinstance(v, dict):
                    lookup.update(create_flat_lookup(v, full_key))
                elif isinstance(v, list):
                    # Store the list as-is
                    lookup[full_key] = v
                else:
                    lookup[full_key] = v
            return lookup
        
        flat_lookup = create_flat_lookup(current_config)
        
        # Process the config with the current flat lookup
        def process_recursive(obj):
            if isinstance(obj, str):
                return substitute_env_vars(obj, flat_lookup)
            elif isinstance(obj, dict):
                return {k: process_recursive(v) for k, v in obj.items()}
            elif isinstance(obj, list):
                return [process_recursive(item) for item in obj]
            else:
                return obj
        
        new_config = process_recursive(current_config)
        
        # If no changes, we're done
        if new_config == current_config:
            break
            
        current_config = new_config
    
    return current_config

def read_yaml_file(file_path: str) -> Dict[str, Any]:
    """读取 YAML 配置文件并替换环境变量
    
    Args:
        file_path: YAML 文件路径
        
    Returns:
        配置字典
        
    Raises:
        FileNotFoundError: 文件不存在
        yaml.YAMLError: YAML 格式错误
        IOError: 文件读取错误
    """
    if not os.path.exists(file_path):
        raise FileNotFoundError(f"配置文件不存在: {file_path}")
    
    try:
        with open(file_path, 'r', encoding='utf-8') as f:
            config_dict = yaml.safe_load(f)
            # 解析 YAML 内容
            return parse_yaml_content(config_dict)
    except yaml.YAMLError as e:
        raise yaml.YAMLError(f"YAML 格式错误: {e}")
    except IOError as e:
        raise IOError(f"文件读取错误: {e}")

def merge_config(config_dict: Dict[str, Any], env_prefix: str = "") -> Dict[str, Any]:
    """合并配置字典和环境变量
    
    Args:
        config_dict: 从文件读取的配置字典
        env_prefix: 环境变量前缀
        
    Returns:
        合并后的配置字典
    """
    merged = {}  # 避免修改原始字典
    
    for key, value in config_dict.items():
        if isinstance(value, dict):
            # 递归处理嵌套字典
            nested_prefix = f"{env_prefix}{key}_" if env_prefix else f"{key}_"
            merged[key] = merge_config(value, nested_prefix)
        else:
            # 优先使用环境变量（如果存在）
            env_key = f"{env_prefix}{key}".upper()
            env_value = get_var(env_key)
            if env_value is not None:
                # 根据原始值类型转换环境变量
                if isinstance(value, bool):
                    merged[key] = env_value.lower() in ('true', '1', 'yes')
                elif isinstance(value, int):
                    try:
                        merged[key] = int(env_value)
                    except ValueError:
                        merged[key] = value  # 转换失败则使用原始值
                elif isinstance(value, float):
                    try:
                        merged[key] = float(env_value)
                    except ValueError:
                        merged[key] = value  # 转换失败则使用原始值
                else:
                    merged[key] = env_value
            else:
                # 环境变量不存在则使用原始值
                merged[key] = value
    
    return merged


def load_config() -> AppConfig:
    """加载配置（主函数）
    
    Returns:
        配置对象
        
    Raises:
        Exception: 配置加载失败
    """
    try:
        startup_file = os.path.abspath(sys.argv[0])
        root_dir = os.path.dirname(startup_file)
        config_path = get_var("FASTAPI_NACOS_CONFIG_FILE", os.path.join(root_dir, "conf", "app.yml"))
        print(f"正在加载配置文件: {config_path}")
        
        # 读取 YAML 配置
        config_dict = read_yaml_file(config_path)
        if config_dict is None:
            config_dict = {}
        
        # 合并环境变量
        # merged_config = merge_config(config_dict)
        
        # 创建配置对象
        return AppConfig(config_dict)
        
    except FileNotFoundError as e:
        print(f"配置文件不存在: {e}")
        print("使用空配置和环境变量初始化...")
        # 文件不存在时使用空配置，所有值从环境变量获取
        return AppConfig({})
        
    except yaml.YAMLError as e:
        print(f"YAML 格式错误: {e}")
        raise RuntimeError(f"配置文件格式错误: {e}")
        
    except IOError as e:
        print(f"文件读取错误: {e}")
        raise RuntimeError(f"配置文件读取失败: {e}")
        
    except Exception as e:
        print(f"配置加载失败: {e}")
        raise RuntimeError(f"配置加载失败: {e}")


if __name__ == "__main__":
    try:
        config = load_config()
        print("配置加载成功!")
        print("\n配置内容:")
        print(config)
        
        # 测试访问方式
        print("\n测试访问方式:")
        # 属性访问
        if hasattr(config, "server"):
            print(f"服务器端口 (属性访问): {config.server.port}")
        # 字典访问
        print(f"服务器端口 (字典访问): {config['server.port']}")
        # get 方法
        print(f"服务器主机 (get方法): {config.get('server.host', 'localhost')}")
        # 环境变量回退
        print(f"环境变量测试: {config.get('test_env_var', '默认值')}")
        
    except Exception as e:
        print(f"测试失败: {e}")
        sys.exit(1)



# 安装: pip install httpx pydantic
from typing import Optional, List
from pydantic import BaseModel
import httpx
from functools import wraps
import asyncio
import inspect

# 定义数据模型
class User(BaseModel):
    id: int
    name: str
    email: str

class Post(BaseModel):
    id: int
    title: str
    content: str

# 声明式客户端装饰器
def service_client(base_url: str):
    def decorator(cls):
        class ServiceClient:
            def __init__(self):
                self.client = httpx.AsyncClient(base_url=base_url)
                
            async def __aenter__(self):
                return self
                
            async def __aexit__(self, *args):
                await self.client.aclose()
        
        for name, method in cls.__dict__.items():
            if callable(method) and not name.startswith('_'):
                setattr(ServiceClient, name, method)
        
        return ServiceClient
    return decorator

# 定义服务接口
@service_client("https://jsonplaceholder.typicode.com")
class UserService:
    """用户服务客户端"""
    
    async def get_user(self, user_id: int) -> User:
        """获取用户信息"""
        response = await self.client.get(f"/users/{user_id}")
        response.raise_for_status()
        return User(**response.json())
    
    async def get_users(self) -> List[User]:
        """获取所有用户"""
        response = await self.client.get("/users")
        response.raise_for_status()
        return [User(**item) for item in response.json()]
    
    async def create_user(self, user: User) -> User:
        """创建用户"""
        response = await self.client.post("/users", json=user.dict())
        response.raise_for_status()
        return User(**response.json())

# 使用示例
async def main():
    async with UserService() as client:
        # 调用声明式方法
        user = await client.get_user(1)
        print(user)
        
        users = await client.get_users()
        print(f"Total users: {len(users)}")

if __name__ == "__main__":
  r = httpx.get("https://www.baidu.com")
  print(r.text)


"""
配置值装饰器模块
"""

import re
from typing import Optional, Callable, Any
from fastapi_nacos.core.manager import NacosClientManager

# 配置值引用正则表达式: ${key:default} 或 ${key}
CONFIG_VALUE_PATTERN = re.compile(r'^\$\{([a-zA-Z0-9_.]+)(?::([^}]*))?\}$')


def Value(config_key: str) -> Callable:
    """
    配置值装饰器，支持从Nacos配置中心获取配置值
    
    支持两种格式：
    1. ${app.name} - 从Nacos获取app.name配置值
    2. ${app.name:api-server} - 从Nacos获取app.name配置值，未找到时返回默认值api-server
    
    Args:
        config_key: 配置值引用字符串，如${app.name}或${app.name:api-server}
        
    Returns:
        装饰器函数
    """
    # 解析配置键和默认值
    match = CONFIG_VALUE_PATTERN.match(config_key)
    if not match:
        raise ValueError(f"Invalid config key format: {config_key}")
    
    key = match.group(1)
    default = match.group(2) if match.group(2) is not None else None
    
    def decorator(func: Callable) -> Callable:
        async def async_wrapper(*args: Any, **kwargs: Any) -> Any:
            # 获取配置值
            config_value = NacosClientManager.get_instance().get_config_val(key, default=default)
            # 直接返回配置值，不调用原始函数
            return config_value
        
        def sync_wrapper(*args: Any, **kwargs: Any) -> Any:
            # 获取配置值
            config_value = NacosClientManager.get_instance().get_config_val(key, default=default)
            # 直接返回配置值，不调用原始函数
            return config_value
        
        # 根据原始函数类型返回对应的包装器
        if asyncio.iscoroutinefunction(func):
            return async_wrapper
        else:
            return sync_wrapper
    
    return decorator


# 导入asyncio用于检测异步函数
import asyncio



from fastapi_nacos.core.manager import NacosClientManager
from fastapi_nacos.core.dependencies import init_nacos_registry_discovery_client, init_nacos_config_client
from contextlib import asynccontextmanager
from fastapi import FastAPI
from fastapi_nacos.models.config import ConfigListener
from fastapi_nacos.utils.log_utils import log
from fastapi_nacos.config import app_config
from fastapi_nacos.utils.ip_utils import get_ip_address

global_ip = get_ip_address()

async def init_nacos_registry_client():
  """初始化Nacos注册中心客户端"""
  discovery_server_addresses = app_config.get("nacos.discovery.server_addresses")
  if not discovery_server_addresses:
    log.warning("nacos.discovery.server_addresses未配置，跳过Nacos注册中心客户端初始化")
  else:
    log.info("开始初始化Nacos注册中心客户端...")
    await init_nacos_registry_discovery_client(
      server_addresses=discovery_server_addresses,
      namespace=app_config.get("nacos.discovery.namespace"),
      username=app_config.get("nacos.discovery.username"),
      password=app_config.get("nacos.discovery.password")
    )
    log.info("Nacos注册中心客户端初始化完成")

async def init_config_client():
  """初始化Nacos配置中心客户端"""
  config_server_addresses = app_config.get("nacos.config.server_addresses")
  if not config_server_addresses:
    log.warning("nacos.config.server_addresses未配置，跳过Nacos配置中心客户端初始化")
  else:
    log.info("开始初始化Nacos配置中心客户端...")
    await init_nacos_config_client(
      server_addresses=config_server_addresses,
      namespace=app_config.get("nacos.config.namespace"),
      username=app_config.get("nacos.config.username"),
      password=app_config.get("nacos.config.password")
    )
    log.info("Nacos配置中心客户端初始化完成")
    log.info("开始初始化Nacos配置中心监听...")
    await init_watch_config()

async def parse_update_config(namespace: str, data_id: str, group: str = "DEFAULT_GROUP", content: str = None):
  """解析并更新Nacos配置"""
  await NacosClientManager.get_instance().parse_config_content(data_id, group, content)
    

async def init_watch_config():
  """初始化Nacos配置中心监听"""
  imports = app_config.get("nacos.config.imports")
  if not imports:
    log.warning("nacos.config.imports未配置，跳过Nacos配置中心监听初始化")
  else:
    log.info("开始初始化Nacos配置中心监听...")
    for item in imports:
      data_id = item.get("data-id")
      group = item.get("group", "DEFAULT_GROUP")
      if not data_id:
        log.warning(f"nacos.config.imports.item.data-id未配置，跳过Nacos配置中心监听初始化: {item}")
        continue
      # 初始化时获取配置
      await NacosClientManager.get_instance().fetch_and_parse_config(data_id=data_id, group=group)
      # 添加监听，当配置变化时变更配置数据
      await NacosClientManager.get_instance().config.add_listener(
        ConfigListener(
          group=group,
          data_id=data_id,
          callback=parse_update_config
        )
      )
    log.info("Nacos配置中心监听初始化完成")

async def startup():
  """自定义启动逻辑"""
  try:
    # 初始化Nacos注册中心客户端
    await init_nacos_registry_client()
    # 初始化Nacos配置中心客户端
    await init_config_client()
    # 注册服务
    app_name = app_config.get("app.name")
    if app_name:
      await NacosClientManager.get_instance().register_service(
        service_name=app_name,
        ip=global_ip,
        port=app_config.get("app.port", 8000),
      )
  except Exception as e:
    log.error(f"服务注册失败: {e}")
    log.info("注意：这可能是因为Nacos服务器未启动或无法连接。测试应用其他功能仍可正常进行。")

async def shutdown():
  """自定义关闭逻辑"""
  try:
    # 注销服务
    app_name = app_config.get("app.name")
    if app_name:
      await NacosClientManager.get_instance().deregister_service(
        service_name=app_name,
        ip=global_ip,
        port=app_config.get("app.port", 8000),
      )

    # 关闭Nacos客户端管理器
    await NacosClientManager.get_instance().config_shutdown()
  except Exception as e:
    log.error(f"服务注销失败: {e}")
    log.info("注意：这可能是因为Nacos服务器未启动或无法连接。测试应用其他功能仍可正常进行。")

@asynccontextmanager
async def nacos_lifespan(app: FastAPI):
    """
    应用生命周期管理器
    - yield 之前：启动逻辑
    - yield 之后：关闭逻辑
    """
    # 启动逻辑
    await startup()
    
    # 应用运行期间
    yield
    
    # 关闭逻辑
    await shutdown()
    
    log.info("应用关闭")

def init_nacos_with_fastapi(app: FastAPI):
  """
  初始化Nacos客户端并注册FastAPI服务
  """
  if app.router.lifespan_context:
      log.warning("FastAPI应用已配置自定义生命周期管理")
      original_lifespan = app.router.lifespan_context
      
      # 包装生命周期管理
      @asynccontextmanager
      async def wrapped_lifespan(app: FastAPI):
        # 执行自定义的nacos生命周期管理
        await startup()
        # 执行原有生命周期管理
        async with original_lifespan(app) as state:
            yield state
        
        # 执行自定义的关闭逻辑
        await shutdown()
      
      app.router.lifespan_context = wrapped_lifespan
      log.info("Nacos生命周期管理已集成到FastAPI应用")
  else:
    log.info("FastAPI应用未配置自定义生命周期管理，将使用Nacos默认生命周期管理")
    app.router.lifespan_context = nacos_lifespan


from typing import Optional
from fastapi import Depends, HTTPException, status
from fastapi_nacos.core import NacosClientManager

# 全局Nacos客户端实例
_nacos_client_manager: Optional[NacosClientManager] = None


async def init_nacos_registry_discovery_client(
    server_addresses: str,
    namespace: str = "public",
    username: Optional[str] = None,
    password: Optional[str] = None
) -> None:
    """
    初始化全局Nacos注册中心发现客户端
    
    Args:
        server_addresses: Nacos服务器地址，格式："ip1:port1"
        namespace: Nacos命名空间ID
        username: Nacos用户名
        password: Nacos密码
    """
    global _nacos_client_manager
    _nacos_client_manager = NacosClientManager.get_instance()
    await _nacos_client_manager.init_registry_discovery_service(
        server_addresses, namespace, username, password
    )

async def init_nacos_config_client(
    server_addresses: str,
    namespace: str = "public",
    username: Optional[str] = None,
    password: Optional[str] = None
) -> None:
    """
    初始化全局Nacos配置中心客户端
    
    Args:
        server_addresses: Nacos服务器地址，格式："ip1:port1"
        namespace: Nacos命名空间ID
        username: Nacos用户名
        password: Nacos密码
    """
    global _nacos_client_manager
    _nacos_client_manager = NacosClientManager.get_instance()
    await _nacos_client_manager.init_config_service(
        server_addresses, namespace, username, password
    )

def get_nacos_client() -> NacosClientManager:
    """
    获取Nacos客户端实例（用于FastAPI依赖注入）
    
    Returns:
        NacosClientManager: Nacos客户端实例
        
    Raises:
        HTTPException: 如果Nacos客户端未初始化
    """
    client = NacosClientManager.get_instance()
    if client is None:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail="Nacos客户端未初始化，请先调用init_nacos_registry_discovery_client或init_nacos_config_client函数"
        )
    return client

def get_nacos_client_no_exception() -> NacosClientManager:
    """
    获取Nacos客户端实例（用于FastAPI依赖注入）
    
    Returns:
        NacosClientManager: Nacos客户端实例
        
    Raises:
        HTTPException: 如果Nacos客户端未初始化
    """
    return NacosClientManager.get_instance()

def get_service_registry():
    """
    获取服务注册管理器
        
    Returns:
        ServiceRegistry: 服务注册管理器
    """
    return get_nacos_client().registry


def get_service_discovery():
    """
    获取服务发现管理器
        
    Returns:
        ServiceDiscovery: 服务发现管理器
    """
    return get_nacos_client().discovery


def get_config_manager():
    """
    获取配置中心管理器
        
    Returns:
        ConfigManager: 配置中心管理器
    """
    return get_nacos_client().config


import json
import yaml
from typing import Dict, Optional
from v2.nacos import NacosConfigService
from fastapi_nacos.models.config import ConfigListener
from fastapi_nacos.utils.exceptions import ConfigError, ConfigListenerError


class ConfigManager:
    """配置中心管理类"""
    
    def __init__(self, config_service: NacosConfigService, logger, server_addresses, namespace, username, password):
        """
        初始化配置中心管理器
        
        Args:
            config_service: Nacos配置服务实例
            logger: 日志记录器
            server_addresses: Nacos服务器地址列表
            namespace: Nacos命名空间
            username: Nacos用户名
            password: Nacos密码
        """
        self.config_service = config_service
        self.logger = logger
        self.server_addresses = server_addresses
        self.namespace = namespace
        self.username = username
        self.password = password
        self.config_listeners: Dict[str, ConfigListener] = {}  # 配置监听器
    
    async def get_config(self, data_id: str, group: str = "DEFAULT_GROUP") -> Optional[str]:
        """
        获取配置信息
        
        Args:
            data_id: 配置数据ID
            group: 配置分组，默认DEFAULT_GROUP
            
        Returns:
            Optional[str]: 配置内容
        """
        try:
            self.logger.info(f"获取配置: data_id={data_id}, group={group}")
            
            # 调用Nacos客户端获取配置
            from v2.nacos.config.model.config_param import ConfigParam
            config_param = ConfigParam(
                data_id=data_id,
                group=group
            )
            content = await self.config_service.get_config(config_param)
            
            self.logger.debug(f"配置获取结果: {content}")
            
            self.logger.info(f"配置获取成功: data_id={data_id}")
            return content
        except Exception as e:
            self.logger.error(f"获取配置失败: data_id={data_id}，错误: {str(e)}")
            raise ConfigError(f"获取配置失败: {str(e)}") from e
    
    async def get_config_dict(self, data_id: str, group: str = "DEFAULT_GROUP") -> Dict:
        """
        获取配置信息并转换为字典
        
        Args:
            data_id: 配置数据ID
            group: 配置分组，默认DEFAULT_GROUP
            
        Returns:
            Dict: 配置内容字典
        """
        content = await self.get_config(data_id, group)
        if not content:
            return {}
        
        try:
            # 尝试解析为JSON
            return json.loads(content)
        except json.JSONDecodeError:
            try:
                # 尝试解析为YAML
                return yaml.safe_load(content)
            except yaml.YAMLError:
                self.logger.error(f"配置内容解析失败: data_id={request.data_id}")
                raise ConfigError(f"配置内容解析失败: data_id={request.data_id}")
    
    async def add_listener(self, listener: ConfigListener) -> bool:
        """
        添加配置监听器
        
        Args:
            listener: 配置监听器模型
            
        Returns:
            bool: 添加是否成功
        """
        try:
            listener_key = f"{self.namespace}:{listener.group}:{listener.data_id}"
            self.logger.info(f"添加配置监听器: {listener_key}")
            
            # 保存监听器
            self.config_listeners[listener_key] = listener
            
            # 在新版本的Nacos SDK中，使用add_listener方法添加监听器
            # 该方法内部会管理监听线程
            await self.config_service.add_listener(
                data_id=listener.data_id,
                group=listener.group,
                listener=listener.callback
            )
            self.logger.info(f"配置监听器添加成功: {listener_key}")
            return True
        except Exception as e:
            self.logger.error(f"添加配置监听器失败: {listener_key}，错误: {str(e)}")
            raise ConfigListenerError(f"添加配置监听器失败: {str(e)}") from e
    
    async def remove_listener(self, data_id: str, group: str = "DEFAULT_GROUP", namespace: str = "") -> bool:
        """
        移除配置监听器
        
        Args:
            data_id: 配置ID
            group: 配置分组
            namespace: 命名空间ID
            
        Returns:
            bool: 移除是否成功
        """
        try:
            listener_key = f"{namespace}:{group}:{data_id}"
            self.logger.info(f"移除配置监听器: {listener_key}")
            
            # 获取监听器
            if listener_key in self.config_listeners:
                listener = self.config_listeners[listener_key]
                
                # 在新版本的Nacos SDK中，使用remove_listener方法移除监听器
                await self.config_service.remove_listener(
                    data_id=listener.data_id,
                    group=listener.group,
                    listener=listener.callback
                )
                
                # 移除监听器
                del self.config_listeners[listener_key]
                self.logger.info(f"配置监听器移除成功: {listener_key}")
                return True
            else:
                self.logger.warning(f"配置监听器不存在: {listener_key}")
                return False
        except Exception as e:
            self.logger.error(f"移除配置监听器失败: {listener_key}，错误: {str(e)}")
            raise ConfigListenerError(f"移除配置监听器失败: {str(e)}") from e
    
    def _start_listener(self, listener: ConfigListener, listener_key: str):
        """
        启动配置监听线程
        
        注意：在新版本的Nacos SDK中，监听器由SDK内部管理，不需要手动启动监听线程
        
        Args:
            listener: 配置监听器模型
            listener_key: 监听器键
        """
        self.logger.info(f"配置监听器 {listener_key} 由Nacos SDK内部管理")
    
    def _stop_listener(self, listener_key: str):
        """
        停止配置监听线程
        
        注意：在新版本的Nacos SDK中，监听器由SDK内部管理，不需要手动停止监听线程
        
        Args:
            listener_key: 监听器键
        """
        self.logger.info(f"配置监听器 {listener_key} 由Nacos SDK内部管理")

    async def shutdown(self):
        """
        关闭配置中心客户端
        """
        if self.config_service:
          await self.config_service.shutdown()


import random
import time
from typing import List, Optional, Dict
from v2.nacos import NacosNamingService
from fastapi_nacos.models.service import ServiceInstance, ServiceInfo
from fastapi_nacos.utils.exceptions import ServiceDiscoveryError


class ServiceDiscovery:
    """服务发现管理类"""
    
    def __init__(self, 
        naming_service: NacosNamingService, 
        logger,
        server_addresses: str,
        namespace: str = "",
        username: Optional[str] = None,
        password: Optional[str] = None):
        """
        初始化服务发现管理器
        
        Args:
            naming_service: Nacos命名服务实例
            logger: 日志记录器
            server_addresses: Nacos服务器地址
            namespace: Nacos命名空间ID
            username: Nacos用户名
            password: Nacos密码
        """
        self.naming_service = naming_service
        self.logger = logger
        self.server_addresses = server_addresses
        self.namespace = namespace
        self.username = username
        self.password = password
        self.service_cache: Dict[str, Dict] = {}  # 服务缓存，格式: {"service_group": {"instances": [], "timestamp": 0}}
        self.cache_ttl = 30  # 缓存有效期，单位：秒
    
    async def get_service_instances(
        self,
        service_name: str,
        group_name: str = "DEFAULT_GROUP",
        healthy_only: bool = True,
        clusters: Optional[List[str]] = None
    ) -> List[ServiceInstance]:
        """
        获取服务实例列表
        
        Args:
            service_name: 服务名称
            group_name: 服务分组
            healthy_only: 是否只返回健康实例
            clusters: 集群列表
            
        Returns:
            List[ServiceInstance]: 服务实例列表
        """
        try:
            self.logger.info(f"查询服务实例: {service_name}，分组: {group_name}，健康实例: {healthy_only}")
            
            # 调用Nacos客户端获取服务实例
            # 使用ListInstanceParam对象
            from v2.nacos.naming.model.naming_param import ListInstanceParam
            list_param = ListInstanceParam(
                service_name=service_name,
                group_name=group_name,
                healthy_only=healthy_only,
                clusters=clusters or []
            )
            instances = await self.naming_service.list_instances(list_param)
            
            self.logger.debug(f"服务实例查询结果: {instances}")
            
            # 转换为ServiceInstance模型列表
            service_instances = []
            # 处理Nacos SDK返回格式，可能是字典或对象
            instances_data = instances
            if hasattr(instances, 'hosts'):
                instances_data = instances.hosts
            elif isinstance(instances, dict) and 'hosts' in instances:
                instances_data = instances['hosts']
            
            for instance in instances_data:
                # 处理实例数据格式，可能是字典或对象
                ip = instance.get('ip') if isinstance(instance, dict) else instance.ip
                port = instance.get('port') if isinstance(instance, dict) else instance.port
                weight = instance.get('weight', 1.0) if isinstance(instance, dict) else instance.weight
                healthy = instance.get('healthy', True) if isinstance(instance, dict) else instance.healthy
                enabled = instance.get('enabled', True) if isinstance(instance, dict) else instance.enabled
                metadata = instance.get('metadata', {}) if isinstance(instance, dict) else instance.metadata
                cluster_name = instance.get('clusterName') if isinstance(instance, dict) else getattr(instance, 'cluster_name', 'DEFAULT')
                instance_id = instance.get('instanceId') if isinstance(instance, dict) else getattr(instance, 'instance_id', f"{ip}:{port}:{cluster_name}@{service_name}")
                
                service_instance = ServiceInstance(
                    ip=ip,
                    port=port,
                    service_name=service_name,
                    group_name=group_name,
                    weight=weight,
                    healthy=healthy,
                    enabled=enabled,
                    metadata=metadata,
                    cluster_name=cluster_name,
                    instance_id=instance_id
                )
                service_instances.append(service_instance)
            
            self.logger.info(f"服务实例查询成功: {service_name}，找到 {len(service_instances)} 个实例")
            return service_instances
        except Exception as e:
            self.logger.error(f"查询服务实例失败: {service_name}，错误: {str(e)}")
            raise ServiceDiscoveryError(f"查询服务实例失败: {str(e)}") from e
    
    async def get_service_info(
        self,
        service_name: str,
        group_name: str = "DEFAULT_GROUP",
        clusters: Optional[List[str]] = None
    ) -> Optional[ServiceInfo]:
        """
        获取完整的服务信息
        
        Args:
            service_name: 服务名称
            group_name: 服务分组
            clusters: 集群列表
            
        Returns:
            Optional[ServiceInfo]: 服务信息模型
        """
        try:
            self.logger.info(f"查询服务信息: {service_name}，分组: {group_name}")
            
            # 调用Nacos客户端获取服务实例
            # 使用ListInstanceParam对象
            from v2.nacos.naming.model.naming_param import ListInstanceParam
            list_param = ListInstanceParam(
                service_name=service_name,
                group_name=group_name,
                healthy_only=False,  # 获取所有实例，无论健康状态
                clusters=clusters or []
            )
            instances = await self.naming_service.list_instances(list_param)
            
            if not instances:
                self.logger.warning(f"未找到服务信息: {service_name}")
                return None
            
            # 转换服务实例
            service_instances = []
            # 处理Nacos SDK返回格式，可能是字典或对象
            instances_data = instances
            if hasattr(instances, 'hosts'):
                instances_data = instances.hosts
            elif isinstance(instances, dict) and 'hosts' in instances:
                instances_data = instances['hosts']
            
            for instance in instances_data:
                # 处理实例数据格式，可能是字典或对象
                ip = instance.get('ip') if isinstance(instance, dict) else instance.ip
                port = instance.get('port') if isinstance(instance, dict) else instance.port
                weight = instance.get('weight', 1.0) if isinstance(instance, dict) else instance.weight
                healthy = instance.get('healthy', True) if isinstance(instance, dict) else instance.healthy
                enabled = instance.get('enabled', True) if isinstance(instance, dict) else instance.enabled
                metadata = instance.get('metadata', {}) if isinstance(instance, dict) else instance.metadata
                cluster_name = instance.get('clusterName') if isinstance(instance, dict) else getattr(instance, 'cluster_name', 'DEFAULT')
                instance_id = instance.get('instanceId') if isinstance(instance, dict) else getattr(instance, 'instance_id', f"{ip}:{port}:{cluster_name}@{service_name}")
                
                service_instance = ServiceInstance(
                    ip=ip,
                    port=port,
                    service_name=service_name,
                    group_name=group_name,
                    weight=weight,
                    healthy=healthy,
                    enabled=enabled,
                    metadata=metadata,
                    cluster_name=cluster_name,
                    instance_id=instance_id
                )
                service_instances.append(service_instance)
            
            # 转换为ServiceInfo模型
            service_info = ServiceInfo(
                name=service_name,
                group_name=group_name,
                clusters=clusters,
                cacheMillis=1000,  # 默认缓存时间
                hosts=service_instances,
                last_ref_time=int(time.time() * 1000),  # 当前时间戳
                checksum=None,
                all_ip=False,
                reach_protection_threshold=None
            )
            
            self.logger.info(f"服务信息查询成功: {service_name}")
            return service_info
        except Exception as e:
            self.logger.error(f"查询服务信息失败: {service_name}，错误: {str(e)}")
            raise ServiceDiscoveryError(f"查询服务信息失败: {str(e)}") from e
    
    async def choose_one_instance(
        self,
        service_name: str,
        group_name: str = "DEFAULT_GROUP",
        healthy_only: bool = True,
        clusters: Optional[List[str]] = None,
        strategy: str = "random"
    ) -> Optional[ServiceInstance]:
        """
        选择一个服务实例（支持负载均衡策略）
        
        Args:
            service_name: 服务名称
            group_name: 服务分组
            healthy_only: 是否只返回健康实例
            clusters: 集群列表
            strategy: 负载均衡策略，可选值: random, round_robin, weight_random
            
        Returns:
            Optional[ServiceInstance]: 选中的服务实例
        """
        try:
            instances = await self.get_service_instances(
                service_name=service_name,
                group_name=group_name,
                healthy_only=healthy_only,
                clusters=clusters
            )
            
            if not instances:
                self.logger.warning(f"未找到可用的服务实例: {service_name}")
                return None
            
            # 根据策略选择实例
            if strategy == "random":
                # 随机选择
                instance = random.choice(instances)
            elif strategy == "weight_random":
                # 加权随机选择
                total_weight = sum(instance.weight for instance in instances)
                if total_weight <= 0:
                    instance = random.choice(instances)
                else:
                    random_val = random.uniform(0, total_weight)
                    current_weight = 0
                    for instance in instances:
                        current_weight += instance.weight
                        if random_val <= current_weight:
                            break
            else:
                # 默认随机选择
                instance = random.choice(instances)
            
            self.logger.info(f"选择服务实例: {service_name}，实例: {instance.ip}:{instance.port}，策略: {strategy}")
            return instance
        except Exception as e:
            self.logger.error(f"选择服务实例失败: {service_name}，错误: {str(e)}")
            raise ServiceDiscoveryError(f"选择服务实例失败: {str(e)}") from e
    
    async def get_all_services(
        self,
        group_name: str = "DEFAULT_GROUP",
        page_no: int = 1,
        page_size: int = 100
    ) -> List[str]:
        """
        获取所有服务名称列表
        
        Args:
            group_name: 服务分组
            page_no: 页码
            page_size: 每页数量
            
        Returns:
            List[str]: 服务名称列表
        """
        try:
            self.logger.info(f"查询所有服务列表，分组: {group_name}")
            
            # 在新版本的Nacos SDK中，获取所有服务的方法名可能不同
            # 这里使用list_services方法，参数可能需要调整
            services = await self.naming_service.list_services(
                group_name=group_name,
                page_no=page_no,
                page_size=page_size
            )
            
            self.logger.debug(f"服务列表查询结果: {services}")
            
            # 提取服务名称
            service_names = []
            if hasattr(services, "service_names"):
                service_names = services.service_names
            elif hasattr(services, "doms"):
                service_names = services.doms
            elif isinstance(services, dict) and "doms" in services:
                service_names = services["doms"]
            
            self.logger.info(f"服务列表查询成功，找到 {len(service_names)} 个服务")
            return service_names
        except Exception as e:
            self.logger.error(f"查询所有服务列表失败，错误: {str(e)}")
            raise ServiceDiscoveryError(f"查询所有服务列表失败: {str(e)}") from e
    
    async def get_instance(
        self,
        service_name: str,
        ip: str,
        port: int,
        group_name: str = "DEFAULT_GROUP",
        cluster_name: str = "DEFAULT"
    ) -> Optional[ServiceInstance]:
        """
        获取单个服务实例信息
        
        Args:
            service_name: 服务名称
            ip: 服务IP地址
            port: 服务端口
            group_name: 服务分组
            cluster_name: 集群名称
            
        Returns:
            Optional[ServiceInstance]: 服务实例信息
        """
        try:
            self.logger.info(f"查询单个服务实例: {service_name}，实例: {ip}:{port}")
            
            # 先获取所有实例，然后筛选
            instances = await self.get_service_instances(
                service_name=service_name,
                group_name=group_name,
                healthy_only=False,
                clusters=[cluster_name]
            )
            
            # 查找匹配的实例
            for instance in instances:
                if instance.ip == ip and instance.port == port:
                    self.logger.info(f"找到服务实例: {service_name}，实例: {ip}:{port}")
                    return instance
            
            self.logger.warning(f"未找到服务实例: {service_name}，实例: {ip}:{port}")
            return None
        except Exception as e:
            self.logger.error(f"查询单个服务实例失败: {service_name}，实例: {ip}:{port}，错误: {str(e)}")
            raise ServiceDiscoveryError(f"查询单个服务实例失败: {str(e)}") from e
    
    async def refresh_service_cache(self, service_name: str, group_name: str = "DEFAULT_GROUP"):
        """
        刷新服务缓存
        
        Args:
            service_name: 服务名称
            group_name: 服务分组
        """
        try:
            self.logger.info(f"刷新服务缓存: {service_name}，分组: {group_name}")
            
            # 获取最新的服务实例
            instances = await self.get_service_instances(
                service_name=service_name,
                group_name=group_name,
                healthy_only=False
            )
            
            # 更新缓存
            cache_key = f"{service_name}@{group_name}"
            self.service_cache[cache_key] = {
                "instances": instances,
                "timestamp": self._get_current_timestamp()
            }
            
            self.logger.info(f"服务缓存刷新成功: {service_name}")
        except Exception as e:
            self.logger.error(f"刷新服务缓存失败: {service_name}，错误: {str(e)}")
            raise ServiceDiscoveryError(f"刷新服务缓存失败: {str(e)}") from e
    
    def _get_current_timestamp(self) -> int:
        """
        获取当前时间戳
        
        Returns:
            int: 当前时间戳（秒）
        """
        import time
        return int(time.time())
    
    def _is_cache_valid(self, cache_key: str) -> bool:
        """
        检查缓存是否有效
        
        Args:
            cache_key: 缓存键
            
        Returns:
            bool: 缓存是否有效
        """
        if cache_key not in self.service_cache:
            return False
        
        cache = self.service_cache[cache_key]
        current_time = self._get_current_timestamp()
        return (current_time - cache["timestamp"]) < self.cache_ttl